# Copyright (C) 2008 Martin Dengler
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import logging
from gettext import gettext as _
from gi.repository import GConf

import glib
from gi.repository import GObject
from gi.repository import Gtk

from sugar3.graphics import style
from sugar3.graphics.icon import get_icon_state, Icon
from sugar3.graphics.tray import TrayIcon
from sugar3.graphics.palette import Palette
from sugar3.graphics.palettemenuitem import PaletteMenuItem
from sugar3.graphics.palettemenuitem import PaletteMenuItemSeparator
from sugar3.graphics.xocolor import XoColor

from jarabe.frame.frameinvoker import FrameWidgetInvoker

#_ICON_NAME = 'display'
_ICON_NAME = 'speaker'


class DeviceView(TrayIcon):

    FRAME_POSITION_RELATIVE = 103

    def __init__(self):
        client = GConf.Client.get_default()
        self._color = XoColor(client.get_string('/desktop/sugar/user/color'))

        TrayIcon.__init__(self, icon_name=_ICON_NAME, xo_color=self._color)

        self.set_palette_invoker(FrameWidgetInvoker(self))

        self._model = DeviceModel()
        self._model.connect('notify::brightness', self.__display_status_changed_cb)
        self._model.connect('notify::monochrome', self.__display_status_changed_cb)

        self.connect('draw', self.__draw_cb)

        self._icon_widget.connect('button-release-event',
                                  self.__button_release_event_cb)

        self._update_info()

    def create_palette(self):
        label = glib.markup_escape_text(_('My Display'))
        palette = DisplayPalette(label, model=self._model)
        palette.set_group_id('frame')
        return palette

    def _update_info(self):
        name = _ICON_NAME
        current_brightness = self._model.props.brightness
        xo_color = self._color

        if self._model.props.monochrome:
            xo_color = XoColor('%s,%s' % (style.COLOR_BLACK.get_svg(),
                                          style.COLOR_WHITE.get_svg()))

        self.icon.props.icon_name = get_icon_state(name, current_brightness,
                                                   step=-1)
        logging.warning('icon_name = %s' % self.icon.props.icon_name)
        self.icon.props.xo_color = xo_color

    def __button_release_event_cb(self, widget, event):
        if event.button != 1:
            return False

        self.palette_invoker.notify_right_click()
        return True

    def __draw_cb(self, *args):
        self._update_info()

    def __display_status_changed_cb(self, pspec_, param_):
        self._update_info()


class DisplayPalette(Palette):

    def __init__(self, primary_text, model):
        Palette.__init__(self, label=primary_text)

        self._model = model

        vbox = Gtk.VBox()
        self.set_content(vbox)
        vbox.show()

        self._monochrome_item = PaletteMenuItem('')
        self._monochrome_icon = Icon(icon_size=Gtk.IconSize.MENU)
        self._monochrome_item.set_image(self._monochrome_icon)
        vbox.add(self._monochrome_item)
        self._monochrome_item.show()
        self._monochrome_item.connect('activate', self.__monochrome_activate_cb)

        separator = PaletteMenuItemSeparator()
        vbox.pack_start(separator, True, True, 0)
        separator.show()

        brightness_step = 25
        self._adjustment = Gtk.Adjustment(value=self._model.props.brightness,
                                          lower=0,
                                          upper=100 + brightness_step,
                                          step_incr=brightness_step,
                                          page_incr=brightness_step,
                                          page_size=brightness_step)
        self._hscale = Gtk.HScale()
        self._hscale.set_adjustment(self._adjustment)
        self._hscale.set_digits(0)
        self._hscale.set_size_request(style.GRID_CELL_SIZE * 4, -1)
        vbox.add(self._hscale)
        self._hscale.show()

        self._adjustment_handler_id = \
            self._adjustment.connect('value_changed',
                                     self.__adjustment_changed_cb)

        self._model_notify_brightness_handler_id = \
            self._model.connect('notify::brightness', self.__brightness_changed_cb)
        self._model.connect('notify::monochrome', self.__monochrome_changed_cb)

        self.connect('popup', self.__popup_cb)

    def _update_monochrome(self):
        if self._model.props.monochrome:
            monochrome_item_text = _('Color')
            monochrome_item_icon_name = 'dialog-ok'
        else:
            monochrome_item_text = _('Monochrome')
            monochrome_item_icon_name = 'dialog-cancel'
        self._monochrome_item.set_label(monochrome_item_text)
        self._monochrome_icon.props.icon_name = monochrome_item_icon_name
        self._monochrome_icon.show()

    def _update_brightness(self):
        if self._adjustment.props.value != self._model.props.brightness:
            self._adjustment.handler_block(self._adjustment_handler_id)
            try:
                self._adjustment.props.value = self._model.props.brightness
            finally:
                self._adjustment.handler_unblock(self._adjustment_handler_id)

    def __adjustment_changed_cb(self, adj_):
        self._model.handler_block(self._model_notify_brightness_handler_id)
        try:
            self._model.props.brightness = self._adjustment.props.value
        finally:
            self._model.handler_unblock(self._model_notify_brightness_handler_id)

    def __brightness_changed_cb(self, pspec_, param_):
        self._update_brightness()

    def __monochrome_activate_cb(self, menuitem_):
        self._model.props.monochrome = not self._model.props.monochrome

    def __monochrome_changed_cb(self, pspec_, param_):
        self._update_monochrome()

    def __popup_cb(self, palette_):
        self._update_brightness()
        self._update_monochrome()


class DeviceModel(GObject.GObject):
    __gproperties__ = {
        'brightness': (int, None, None, 0, 100, 0, GObject.PARAM_READWRITE),
        'monochrome': (bool, None, None, False, GObject.PARAM_READWRITE),
    }

    def __init__(self):
        GObject.GObject.__init__(self)

        # TODO
        #sound.monochrome_changed.connect(self.__monochrome_changed_cb)
        #sound.brightness_changed.connect(self.__brightness_changed_cb)
        self._set_brightness(100)
        self._set_monochrome(False)

    def __monochrome_changed_cb(self, **kwargs):
        self.notify('monochrome')

    def __brightness_changed_cb(self, **kwargs):
        self.notify('brightness')

    def _get_brightness(self):
        # TODO
        #return sound.get_volume()
        return self.brightness

    def _set_brightness(self, new_brightness):
        # TODO
        #sound.set_volume(new_volume)
        self.brightness = new_brightness

    def _get_monochrome(self):
        # TODO
        #return sound.get_muted()
        return self.monochrome

    def _set_monochrome(self, monochrome):
        # TODO
        #sound.set_muted(mute)
        self.monochrome = monochrome

    def get_type(self):
        return 'display'

    def do_get_property(self, pspec):
        if pspec.name == 'brightness':
            return self._get_brightness()
        elif pspec.name == 'monochrome':
            return self._get_monochrome()

    def do_set_property(self, pspec, value):
        if pspec.name == 'brightness':
            self._set_brightness(value)
        elif pspec.name == 'monochrome':
            self._set_monochrome(value)


def setup(tray):
    tray.add_device(DeviceView())
